-- GridStatusSanity.lua
--
-- Created By : Greltok

--{{{ Libraries
local L = AceLibrary("AceLocale-2.2"):new("GridStatusSanity")
local GridRoster = Grid:GetModule("GridRoster")
--}}}

local sanityName = GetSpellInfo(63050)

local GridStatusSanity = Grid:GetModule("GridStatus"):NewModule("Sanity")
GridStatusSanity.menuName = L["Sanity"]

--{{{ AceDB defaults
GridStatusSanity.defaultDB = {
    debug = false,
    sanity = {
        text = L["Sanity"],
        enable = true,
        color = { r = 0.88, g = 0.62, b = 0.38, a = 1 },
        priority = 90,
        range = false,
        threshold = 50,
    },
}
--}}}

GridStatusSanity.options = false

--{{{ additional options
local sanityOptions = {
    ["threshold"] = {
        type = "range",
        name = L["Sanity threshold"],
        desc = L["Set the minimum sanity for the low sanity warning."],
        max = 99,
        min = 1,
        step = 1,
        get = function()
            return GridStatusSanity.db.profile.sanity.threshold
        end,
        set = function(v)
            GridStatusSanity.db.profile.sanity.threshold = v
            GridStatusSanity:SanityCheckAll()
        end,
    },
}
--}}}

function GridStatusSanity:OnInitialize()
    self.super.OnInitialize(self)
    self:RegisterStatus("sanity", L["Sanity"], sanityOptions, true)
end

function GridStatusSanity:OnStatusEnable(status)
	if status == "sanity" then
		self:RegisterEvent("Grid_UnitJoined")
		self:RegisterEvent("UNIT_AURA", "SanityCheck")
		self:SanityCheckAll()
	end
end

function GridStatusSanity:OnStatusDisable(status)
	if status == "sanity" then
		self:UnregisterEvent("Grid_UnitJoined")
		self:UnregisterEvent("UNIT_AURA")
		self.core:SendStatusLostAllUnits("sanity")
	end
end

function GridStatusSanity:Reset()
	self.super.Reset(self)
	self:SanityCheckAll()
end

function GridStatusSanity:Grid_UnitJoined(guid, unitid)
	self:SanityCheck(unitid)
end

function GridStatusSanity:SanityCheckAll()
	for guid, unitid in GridRoster:IterateRoster() do
		self:SanityCheck(unitid)
	end
end

function GridStatusSanity:SanityCheck(unitid)
	local guid = UnitGUID(unitid)
	if not GridRoster:IsGUIDInRaid(guid) then
		return
	end
	local gained
	local name, _, icon, count = UnitDebuff(unitid, sanityName)
	if name then
		local settings = self.db.profile.sanity
		if count <= settings.threshold then
			self.core:SendStatusGained(guid,
										"sanity",
										settings.priority,
										(settings.range and 40),
										settings.color,
										settings.text,
										count,
										nil,
										icon,
										nil,
										nil,
										count)
			gained = true
		end
	end
	if not gained then
		self.core:SendStatusLost(guid, "sanity")
	end
end
